@@ -7,6 +7,7 @@ module Text.ParserCombinators.Poly.Plain
   , next	-- :: Parser t t
   , eof		-- :: Parser t ()
   , satisfy	-- :: (t->Bool) -> Parser t t
+  , satisfyMsg	-- :: (t->Bool) -> String -> Parser t t
   , onFail	-- :: Parser t a -> Parser t a -> Parser t a
 
     -- ** Re-parsing
@@ -28,17 +29,4 @@ import Control.Applicative
 runParser :: Parser t a -> [t] -> (Either String a, [t])
 runParser (P p) = resultToEither . p
 
-instance Applicative (Parser t) where
-    pure f    = return f
-    pf <*> px = do { f <- pf; x <- px; return (f x) }
-#if defined(GLASGOW_HASKELL) && GLASGOW_HASKELL > 610
-    p  <*  q  = p `discard` q
-#endif
-
-instance Alternative (Parser t) where
-    empty     = fail "no parse"
-    p <|> q   = p `onFail` q
-
-instance PolyParse (Parser t)
-
 ------------------------------------------------------------------------
